@@ -4,19 +4,8 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#include "runnermanager.h"
-
-#ifdef KSERVICE_BUILD_DEPRECATED_SINCE
-#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72) && KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
-#define WITH_KSERVICE 1
-#endif
-#endif
-
-#ifndef WITH_KSERVICE
-#define WITH_KSERVICE 0
-#endif
-
-#include <QAction>
+#include <KRunner/Action>
+#include <KRunner/RunnerManager>
 #include <QObject>
 #include <QProcess>
 #include <QSignalSpy>
@@ -24,18 +13,15 @@
 #include <QTest>
 #include <QTime>
 #include <QTimer>
-
-#if WITH_KSERVICE
-#include <KSycoca>
-#endif
+#include <memory>
 
 #include "abstractrunnertest.h"
 #include "kpluginmetadata_utils_p.h"
 
-using namespace Plasma;
+using namespace KRunner;
 
-Q_DECLARE_METATYPE(Plasma::QueryMatch)
-Q_DECLARE_METATYPE(QList<Plasma::QueryMatch>)
+Q_DECLARE_METATYPE(KRunner::QueryMatch)
+Q_DECLARE_METATYPE(QList<KRunner::QueryMatch>)
 
 class DBusRunnerTest : public AbstractRunnerTest
 {
@@ -45,7 +31,6 @@ public:
     ~DBusRunnerTest() override;
 
 private Q_SLOTS:
-    void initTestCase();
     void cleanup();
     void testMatch();
     void testMulti();
@@ -55,32 +40,18 @@ private Q_SLOTS:
     void testDBusRunnerSyntaxIntegration();
     void testIconData();
     void testLifecycleMethods();
-    void testRequestActionsOnceWildcards();
-#if WITH_KSERVICE
-    void testMulti_data();
-#endif
+    void testRequestActionsWildcards();
 };
 
 DBusRunnerTest::DBusRunnerTest()
     : AbstractRunnerTest()
 {
-    qRegisterMetaType<QList<Plasma::QueryMatch>>();
-}
-
-DBusRunnerTest::~DBusRunnerTest()
-{
-}
-
-void DBusRunnerTest::initTestCase()
-{
-    // Set up a layer in the bin dir so ksycoca & KPluginMetaData find the Plasma/Runner service type
-    const QByteArray defaultDataDirs = qEnvironmentVariableIsSet("XDG_DATA_DIRS") ? qgetenv("XDG_DATA_DIRS") : QByteArray("/usr/local:/usr");
-    const QByteArray modifiedDataDirs = QFile::encodeName(QCoreApplication::applicationDirPath()) + QByteArrayLiteral("/data:") + defaultDataDirs;
-    qputenv("XDG_DATA_DIRS", modifiedDataDirs);
-
+    qRegisterMetaType<QList<KRunner::QueryMatch>>();
     QStandardPaths::setTestModeEnabled(true);
 }
 
+DBusRunnerTest::~DBusRunnerTest() = default;
+
 void DBusRunnerTest::cleanup()
 {
     // Make sure kill the running processes after each test
@@ -91,10 +62,9 @@ void DBusRunnerTest::testMatch()
 {
     QProcess *process = startDBusRunnerProcess({QStringLiteral("net.krunnertests.dave")});
     initProperties();
-    launchQuery(QStringLiteral("foo"));
+    const auto matches = launchQuery(QStringLiteral("foo"));
 
     // verify matches
-    const QList<QueryMatch> matches = manager->matches();
     QCOMPARE(matches.count(), 1);
     auto result = matches.first();
 
@@ -102,63 +72,38 @@ void DBusRunnerTest::testMatch()
     QCOMPARE(result.id(), QStringLiteral("dbusrunnertest_id1")); // note the runner name is prepended
     QCOMPARE(result.text(), QStringLiteral("Match 1"));
     QCOMPARE(result.iconName(), QStringLiteral("icon1"));
-    QCOMPARE(result.type(), Plasma::QueryMatch::ExactMatch);
+    QCOMPARE(result.categoryRelevance(), qToUnderlying(KRunner::QueryMatch::CategoryRelevance::Highest));
     QCOMPARE(result.isMultiLine(), true);
     // relevance can't be compared easily because RunnerContext meddles with it
 
     // verify actions
-    QTRY_COMPARE_WITH_TIMEOUT(manager->actionsForMatch(result).count(), 1, 10000);
-    auto action = manager->actionsForMatch(result).constFirst();
+    QCOMPARE(result.actions().size(), 1);
+    auto action = result.actions().constFirst();
 
-    QCOMPARE(action->text(), QStringLiteral("Action 1"));
+    QCOMPARE(action.text(), QStringLiteral("Action 1"));
 
     QSignalSpy processSpy(process, &QProcess::readyRead);
     manager->run(result);
     processSpy.wait();
     QCOMPARE(process->readAllStandardOutput().trimmed().split('\n').constLast(), QByteArray("Running:id1:"));
 
-    result.setSelectedAction(action);
-    manager->run(result);
+    manager->run(result, action);
     processSpy.wait();
     QCOMPARE(process->readAllStandardOutput().trimmed().split('\n').constLast(), QByteArray("Running:id1:action1"));
 }
 
-#if WITH_KSERVICE
-void DBusRunnerTest::testMulti_data()
-{
-    QTest::addColumn<bool>("useKService");
-
-    QTest::newRow("deprecated") << true;
-    QTest::newRow("non-deprecated") << false;
-}
-#endif
-
 void DBusRunnerTest::testMulti()
 {
-#if WITH_KSERVICE
-    QFETCH(bool, useKService);
-#endif
     startDBusRunnerProcess({QStringLiteral("net.krunnertests.multi.a1")}, QStringLiteral("net.krunnertests.multi.a1"));
     startDBusRunnerProcess({QStringLiteral("net.krunnertests.multi.a2")}, QStringLiteral("net.krunnertests.multi.a2"));
-    manager.reset(new RunnerManager()); // This case is special, because we want to load the runners manually
+    manager = std::make_unique<RunnerManager>(); // This case is special, because we want to load the runners manually
 
-#if WITH_KSERVICE
-    if (useKService) {
-        KService::Ptr s(new KService(QFINDTESTDATA("dbusrunnertestmulti.desktop")));
-        QVERIFY(s);
-        manager->loadRunner(s);
-    } else {
-#endif
-        auto md = parseMetaDataFromDesktopFile(QFINDTESTDATA("dbusrunnertestmulti.desktop"));
-        QVERIFY(md.isValid());
-        manager->loadRunner(md);
-#if WITH_KSERVICE
-    }
-#endif
-    launchQuery(QStringLiteral("foo"));
+    auto md = parseMetaDataFromDesktopFile(QFINDTESTDATA("plugins/dbusrunnertestmulti.desktop"));
+    QVERIFY(md.isValid());
+    manager->loadRunner(md);
+    const auto matches = launchQuery(QStringLiteral("foo"));
 
     // verify matches, must be one from each
-    const QList<QueryMatch> matches = manager->matches();
     QCOMPARE(matches.count(), 2);
 
     const QString first = matches.at(0).data().toList().constFirst().toString();
@@ -170,19 +115,17 @@ void DBusRunnerTest::testMulti()
 
 void DBusRunnerTest::testRequestActionsOnce()
 {
-    startDBusRunnerProcess({QStringLiteral("net.krunnertests.dave")});
+    QProcess *process = startDBusRunnerProcess({QStringLiteral("net.krunnertests.dave")});
     initProperties();
 
-    // Construct a fake match with necessary data
-    QueryMatch fakeMatch(runner);
-    fakeMatch.setId(QStringLiteral("dbusrunnertest_id1"));
-    fakeMatch.setData(QVariantList({QStringLiteral("net.krunnertests.dave"), QStringList({QStringLiteral("action1"), QStringLiteral("action2")})}));
-
-    // The actions should not be fetched before we have set up the match session
-    QCOMPARE(manager->actionsForMatch(fakeMatch).count(), 0);
     launchQuery(QStringLiteral("foo"));
-    // We need to retry this, because the DBus call to fetch the actions is async
-    QTRY_COMPARE_WITH_TIMEOUT(manager->actionsForMatch(fakeMatch).count(), 2, 2500);
+    QVERIFY(!manager->matches().constFirst().actions().isEmpty());
+    manager->matchSessionComplete();
+    launchQuery(QStringLiteral("fooo"));
+    const QString processOutput(process->readAllStandardOutput());
+    QCOMPARE(processOutput.count("Matching"), 2);
+    QCOMPARE(processOutput.count("Actions"), 1);
+    QVERIFY(!manager->matches().constFirst().actions().isEmpty());
 }
 
 void DBusRunnerTest::testFilterProperties_data()
@@ -208,7 +151,7 @@ void DBusRunnerTest::testFilterProperties()
     QVERIFY(process->readAllStandardOutput().isEmpty());
     // accepted query fits those constraints
     launchQuery(acceptedQuery);
-    QCOMPARE(process->readAllStandardOutput().trimmed(), QString(QStringLiteral("Matching:") + acceptedQuery).toLocal8Bit());
+    QCOMPARE(QString(process->readAllStandardOutput()).remove("Actions").trimmed(), QStringLiteral("Matching:") + acceptedQuery);
 }
 
 void DBusRunnerTest::testDBusRunnerSyntaxIntegration()
@@ -231,9 +174,7 @@ void DBusRunnerTest::testIconData()
     startDBusRunnerProcess({QStringLiteral("net.krunnertests.dave")});
     initProperties();
 
-    launchQuery(QStringLiteral("fooCostomIcon"));
-
-    const auto matches = manager->matches();
+    const auto matches = launchQuery(QStringLiteral("fooCostomIcon"));
     QCOMPARE(matches.count(), 1);
     auto result = matches.first();
 
@@ -247,8 +188,8 @@ void DBusRunnerTest::testIconData()
 void DBusRunnerTest::testLifecycleMethods()
 {
     QProcess *process = startDBusRunnerProcess({QStringLiteral("net.krunnertests.dave"), QString()});
-    manager.reset(new RunnerManager()); // This case is special, because we want to load the runners manually
-    auto md = parseMetaDataFromDesktopFile(QFINDTESTDATA("dbusrunnertestruntimeconfig.desktop"));
+    manager = std::make_unique<RunnerManager>(); // This case is special, because we want to load the runners manually
+    auto md = parseMetaDataFromDesktopFile(QFINDTESTDATA("plugins/dbusrunnertestruntimeconfig.desktop"));
     manager->loadRunner(md);
     QCOMPARE(manager->runners().count(), 1);
     // Match session should be set up automatically
@@ -262,6 +203,7 @@ void DBusRunnerTest::testLifecycleMethods()
     const QStringList lifeCycleSteps = QString::fromLocal8Bit(process->readAllStandardOutput()).split(QLatin1Char('\n'), Qt::SkipEmptyParts);
     const QStringList expectedLifeCycleSteps = {
         QStringLiteral("Config"),
+        QStringLiteral("Actions"),
         QStringLiteral("Matching:fooo"),
         QStringLiteral("Teardown"),
     };
@@ -275,29 +217,22 @@ void DBusRunnerTest::testLifecycleMethods()
     QVERIFY(manager->matches().isEmpty());
 }
 
-void DBusRunnerTest::testRequestActionsOnceWildcards()
+void DBusRunnerTest::testRequestActionsWildcards()
 {
     initProperties();
-    manager.reset(new RunnerManager()); // This case is special, because we want to load the runners manually
-    auto md = parseMetaDataFromDesktopFile(QFINDTESTDATA("dbusrunnertestmulti.desktop"));
+    manager = std::make_unique<RunnerManager>(); // This case is special, because we want to load the runners manually
+    auto md = parseMetaDataFromDesktopFile(QFINDTESTDATA("plugins/dbusrunnertestmulti.desktop"));
     QVERIFY(md.isValid());
     manager->loadRunner(md);
     QCOMPARE(manager->runners().count(), 1);
-    launchQuery("foo");
-    QVERIFY(manager->matches().isEmpty());
-    QueryMatch match(manager->runners().constFirst());
-    match.setId("test");
-    match.setData(QStringList{"net.krunnertests.multi.a1"});
-    QVERIFY(manager->actionsForMatch(match).isEmpty());
-    manager->matchSessionComplete();
 
-    // We have started the process later and the actions should now be fetched when the match session is started
     startDBusRunnerProcess({QStringLiteral("net.krunnertests.multi.a1")}, QStringLiteral("net.krunnertests.multi.a1"));
-    QTest::qWait(500); // Wait a bit for the runner to pick up the new service
+    startDBusRunnerProcess({QStringLiteral("net.krunnertests.multi.a2")}, QStringLiteral("net.krunnertests.multi.a2"));
+    const auto matches = launchQuery("foo");
+    QCOMPARE(matches.count(), 2);
 
-    launchQuery("fooo");
-    QVERIFY(!manager->matches().isEmpty());
-    QVERIFY(!manager->actionsForMatch(match).isEmpty());
+    QCOMPARE(matches.at(0).actions().count(), 1);
+    QCOMPARE(matches.at(0).actions(), matches.at(1).actions());
 }
 
 QTEST_MAIN(DBusRunnerTest)
